Package: spelling
Title: Tools for Spell Checking in R
Version: 2.3.2
Authors@R: c(
    person("Jeroen", "Ooms", , "jeroenooms@gmail.com", role = c("cre", "aut"),
      comment = c(ORCID = "0000-0002-4035-0289")),
    person("Jim", "Hester", , "james.hester@rstudio.com", role = "aut"))
Description: Spell checking common document formats including latex, markdown, manual pages,
    and description files. Includes utilities to automate checking of documentation and 
    vignettes as a unit test during 'R CMD check'. Both British and American English are 
    supported out of the box and other languages can be added. In addition, packages may
    define a 'wordlist' to allow custom terminology without having to abuse punctuation.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://ropensci.r-universe.dev/spelling
        https://docs.ropensci.org/spelling/
BugReports: https://github.com/ropensci/spelling/issues
Imports: commonmark, xml2, hunspell (>= 3.0), knitr
Suggests: pdftools
RoxygenNote: 7.3.1
Language: en-GB
NeedsCompilation: no
Packaged: 2025-08-18 18:41:13 UTC; jeroen
Author: Jeroen Ooms [cre, aut] (ORCID: <https://orcid.org/0000-0002-4035-0289>),
  Jim Hester [aut]
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2025-08-18 22:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 16:40:13 UTC; unix
