# Generated by roxygen2: do not edit by hand

S3method(print,summary_spellcheck)
export(get_wordlist)
export(spell_check_files)
export(spell_check_package)
export(spell_check_setup)
export(spell_check_test)
export(spell_check_text)
export(update_wordlist)
